class QuickVote < Election
  after_validation :create_candidates
  validates_uniqueness_of :name
  validates_presence_of :name
  attr_accessor :raw_candidates
  attr_accessor :reviewed
  
  def validate
    if not @raw_candidates or @raw_candidates.length < 2
      errors.add(nil, "You must list at least two candidates.")
    end
    
    @raw_candidates.each do |c|
      unless c.instance_of? String
        errors.add(nil, "Candidates must be strings")
        next
      end
      c.strip!
      if c.length == 0
        errors.add(nil, "Candidate name must not be empty")
        next
      end
    end if @raw_candidates

    errors.add(nil, "Candidates must all be unique") if @raw_candidates and @raw_candidates.uniq!

    if name =~ /[^A-Za-z0-9]/
      errors.add(:name, "must only include numbers and letters.")
    end
    if name =~ /^[0-9]+$/
      errors.add(:name, "must not be a number")
    end
    
    if name =~ /^(create|index|confirm|change|results)$/
      errors.add(:name, " is a reserved word.")
    end
  end
  
  def initialize(params={})
    super
    self.startdate = Time.now
    self.enddate =  Time.now + 30.days
    self.active = 1
    self.anonymous = 1
  end

  def candidatelist=(candlist)
    @raw_candidates = candlist
  end

  def name
    read_attribute( :name ).downcase() if read_attribute( :name )
  end

  def reviewed?
    reviewed.to_i == 1
  end

  def create_candidates
    return unless errors.empty?
    @raw_candidates.each do |name|
      candidate = Candidate.new({:name => name})
      self.candidates << candidate
    end
  end

  ### Convert a shortname or id into a QuickVote
  def self.ident_to_quickvote(ident)
    return nil unless ident
    if ident.match(/^\d+$/)
      quickvote = QuickVote.find(ident)
    else
      quickvote = QuickVote.find(:all, :conditions => ["name = ?", ident])[0]
    end

    return quickvote
  end
end
